<?php
// Class CSE_display.
// (cc) CSE, 2008
//
//
// Version Changes
// 
// 22.01.2008 -- added bitfield for buttons in Infopanels, individual button control.
// Released 20.01.2008
// 08.08.2008 -- added galeries
// 10.08.2008 -- fixes for compatibility with CSE_CMS
// 07.01.2009 -- release version with CSE_CMS 1.0

// =====================================================
// Global variables:

// Main page of your project:
$main_page = "../index.php";

// =====================================================

// Classes description:

class CSE_buffer {

	// =====================================================
	// Buffer
	// =====================================================
	// This is separate as a convenience feature. When you output something,
	// but don't now yet if something else will come before, or where you
	// want to output it, store it in a buffer and dump it later.
	// 
	// Use the default buffer without specifying an ID:
	// $mybuffer = new CSE_buffer();
	// $mybuffer->append("foo");
	// $mybuffer->append(" bar");
	// $mybuffer->prepend("Nuff said: ");
	// $mybuffer->dump; // returns "Nuff said: foo bar"
	//
	// Use multiple buffers with ID specification (default buffer remains available):
	// $mybuffer = new CSE_buffer();
	// $mybuffer->append("foo");
	// $id = $mybuffer->newBuffer("I said:"); // get the ID
	// $mybuffer->prepend("Nuff said: ");
	// $mybuffer->append(" bar", $id); // use the ID
	// $mybuffer->dump($id); // use the ID, returns "I said: bar"
	// $mybuffer->dump; // returns "Nuff said: foo"
	
	var $buffer; // an array of buffers
	
	// creator prepares a first empty buffer with ID = 0
	function CSE_buffer() {
		$this->buffer[] = "";
	}
	
	// create a new buffer space, return an ID, optionally fill already with
	// string given.
	function newBuffer($txt = "") {
		$this->buffer[] = $txt;
		return count($this->buffer);
	}
	
	// set buffer value (erase previous content)
	function set($txt, $id=0) {
		$this->buffer[$id] = $txt;
	}
	
	// append in buffer
	function append($txt, $id=0) {
		$this->buffer[$id] .= $txt;
	}
	
	// prepend in buffer
	function prepend($txt, $id=0) {
		$this->buffer[$id] = $txt . $this->buffer[$id];
	}
	
	// dump buffer
	function dump($id=0) {
		return $this->buffer[$id];
	}
	
	// does buffer with ID exist?
	function exist($id) {
		if ($id < count($this->buffer)) {
			return 1;
		} else {
			return 0;
		}
	}
} // END CLASS CSE_buffer

class CSE_gallery {

	// =====================================================
	// Gallery
	// =====================================================
	// This is a class to organize picture collection, based
	// on directories on your server.
	// Nested folders are displayed as "chapters".
	// Folders containing pictures are displayed as a thumbnail
	// collection liked with the original images in a new window. 
	// If necessary, the class will generate thumbnails.

	var $buffer;
	var $folder;
	var $title;
	var $img_per_line;
	var $tmbsz;
	var $tnquality;
	var $displayinfo;
	var $main_page;
	var $ischapter;
	var $curPath;
	var $curPathBack;
	var $introTxt;
	var $imgRoot;

	function CSE_gallery($dir, $title="") {
		$this->folder = $dir;
		$this->buffer = new CSE_buffer;
		$this->title = $title;
		$this->img_per_line = 3;
		$this->tmbsz = 128;
		$this->tnquality = 90;
		$this->displayinfo = 0;
		$this->introTxt = "";
		$this->curPath = "";
		$this->curPathBack = "";
		$this->imgRoot = $this->folder;
		$this->ischapter = false;
		// I assume the main page is just index.php relative to the url of the function that dumps finally the object.
		$this->main_page = "index.php"; // this should be this file;
		if ($title == "") {
			$title = $dir;
		}
		// Now we need to check whether we are in a subfolder:
		if (isset($_GET["CSE_Gallery_Chapter"])) {
			$this->folder = $dir."/".$_GET["CSE_Gallery_Chapter"];
			$this->title = $this->title .": ". $_GET["CSE_Gallery_Chapter"];
			$this->ischapter = true;
			$this->curPath = $_GET["CSE_Gallery_Chapter"]."/";
			$tmp = split("\/",$_GET["CSE_Gallery_Chapter"]);
			if (count($tmp)>1) {
				for ($i=0;$i<(count($tmp)-1);$i++) {
					$this->curPathBack .= $tmp[$i];
				}
			}
		}
	}
	
	function setMainPage($url) {
		$this->main_page = $url;
	}
	
	function createThumbnail($image_file, $type, $size, $border=false) {
		switch($type)
		{
			case "jpg":
				$img = ImageCreateFromJpeg($image_file);
				break;
			case "jpeg":
				$img = ImageCreateFromJpeg($image_file);
				break;
			case "gif":
				$img = ImageCreateFromGif($image_file);
				break;
			case "png":
				$img = ImageCreateFromPng($image_file);
				break;
		}
		if ($img) {
			$x = imagesx($img);
			$y = imagesy($img);
			$szratio = $x/$y;
			if ($szratio > 1) {
				$newx = $size;
				$newy = round($newx/$szratio);
			} else {
				$newy = $size;
				$newx = round($newy*$szratio);
			}
			
			$tn_img=imagecreatetruecolor($newx,$newy);
			if ($border) {
				ImageCopyResampled($tn_img,$img,1,1,0,0,$newx-2,$newy-2,$x,$y);
			} else {
				ImageCopyResampled($tn_img,$img,0,0,0,0,$newx,$newy,$x,$y);
			}
		}
		unset($img);
		return $tn_img;
	}
	
	function makeTn($images, $images_tn, $images_nme, $folders, $folders_tn, $folders_nme) {
		if (mkdir($this->folder."/tn")) {
			// make thumbnails for the images
			for ($i = 0;$i<count($images);$i++) {
				// find image type
				$nms = split("\.",$images_nme[$i]);
				$type = strtolower($nms[count($nms)-1]);
				$tn_img = $this->createThumbnail($images[$i], $type, $this->tmbsz);
				if ($tn_img) {
					switch($type)
					{
						case "jpg":
							imageJpeg($tn_img,$images_tn[$i],$this->tnquality);
							break;
						case "jpeg":
							imageJpeg($tn_img,$images_tn[$i],$this->tnquality);
							break;
						case "gif":
							imageGif($tn_img,$images_tn[$i],$this->tnquality);
							break;
						case "png":
							imagePng($tn_img,$images_tn[$i],$this->tnquality);
							break;
					}
					unset($tn_img);
				} else {
					$this->buffer->append("Error: Gallery cannot create an image for: ".$images_nme[$i]);
				}
				
			}
			// make tn for folders - much more fun...
			for ($i = 0;$i<count($folders);$i++) {
				$all = opendir($folders[$i]);
				$sub_images = array();
				while ($file = readdir($all)) {
					if (preg_match("/\.(jpg|jpeg|png|gif)$/i", $file)) {
						array_push($sub_images,$folders[$i]."/".$file);
					}
				}
				closedir($all);
				unset($all);
				
				$tn_img=imagecreatetruecolor($this->tmbsz,$this->tmbsz);
				imagesavealpha($tn_img, true);
				imagealphablending($tn_img, false);
				$transp_col = imagecolorallocatealpha($tn_img,111,145, 145, 127);
				imagefill($tn_img, 0, 0, $transp_col);
				imagecolortransparent($tn_img, $transp_col);
				
				shuffle($sub_images);
				$offset = 1;
				for ($j=0;$j<(count($sub_images));$j++) {
					$nms = split("\.",$sub_images[$j]);
					$type = strtolower($nms[count($nms)-1]);
					$tmp_img = $this->createThumbnail($sub_images[$j], $type, $this->tmbsz-(count($sub_images)*3)+1, true);
					$x = imagesx($tmp_img);
					$y = imagesy($tmp_img);
					ImageCopyMerge($tn_img,$tmp_img,$offset,$this->tmbsz-$offset-$y,0,0,$x,$y,30+70*$j/count($sub_images));
					$offset = $offset + 3;
				}			
				
				imagePng($tn_img,$folders_tn[$i]);
				
				unset($sub_images);
			}
		} else {
			$this->buffer->append("Error: Gallery cannot create ".$this->folder."/tn folder! ".$php_errormsg);
		}
	}
	
	function setImageRoot($root) {
		$this->imgRoot = $root;
	}
	
	function makeGallery() {
		// get the files in the folder
		if (!$this->folder or !is_dir($this->folder)) {
			$this->buffer->append("Error: Gallery must be given a folder, was given: ".$this->folder);
			return;
		}
		if (isset($_GET["CSE_Gallery_Chapter"])) {
			$this->imgRoot .= "/".$_GET["CSE_Gallery_Chapter"];
		}
		$all = opendir($this->folder);
		$folders = array();
		$folders_nme = array();
		$folders_tn = array();
		$folders_tn_url = array();
		$images = array();
		$images_url = array();
		$images_nme = array();
		$images_tn = array();
		$images_tn_url = array();
		$hastn = 0;
		while ($file = readdir($all)) {
			if (is_dir($this->folder."/".$file) and $file <> ".." and $file <> ".") {
				if ($file == "tn") {
					$hastn = 1;
				} else {
					array_push($folders,$this->folder."/".$file);
					array_push($folders_nme,$file);
					array_push($folders_tn,$this->folder."/tn/".$file.".png");
					array_push($folders_tn_url,$this->imgRoot."/tn/".$file.".png");
				}
			} else {
				if (preg_match("/\.(jpg|jpeg|png|gif)$/i", $file)) {
					array_push($images,$this->folder."/".$file);
					array_push($images_url,$this->imgRoot."/".$file);
					array_push($images_tn,$this->folder."/tn/".$file);
					array_push($images_tn_url,$this->imgRoot."/tn/".$file);
					array_push($images_nme,$file);
				} else {
					if ($file == "intro.txt") {
						$this->introTxt = file_get_contents($this->folder."/intro.txt");
					}
				}
			}
		}
		
		closedir($all);
		unset($all);
		
		if ($this->displayinfo) {
			$this->buffer->append("<span class=gallery_info>");
			$this->buffer->append("Has tn: ".$hastn."<br>");
			$this->buffer->append("Chapters: ".count($folders)."<br>");
			$this->buffer->append("Images: ".count($images)."<br>");
			$this->buffer->append("</span><br>");
		}
		
		// if not tn folder is found, create it, and populate it. This requires gd...
		if ($hastn == 0) {
			$this->makeTn($images, $images_tn, $images_nme, $folders, $folders_tn, $folders_nme);
		}
		
		// Make the gallery
		$this->buffer->append("<table class=gallery><tr>");
		if ($this->ischapter) {
			if ($this->curPathBack == "") {
				$this->buffer->append("<td class=gallery_td_title colspan=".$this->img_per_line."><span class=gallery_title>".$this->title."</span>&nbsp;&nbsp;<span class=gallery_back><b>[</b><a href=".$this->main_page.">Back</a><b>]</b></span>");
			} else {
				$this->buffer->append("<td class=gallery_td_title colspan=".$this->img_per_line."><span class=gallery_title>".$this->title."</span>&nbsp;&nbsp;<span class=gallery_back><b>[</b><a href=".$this->main_page."?CSE_Gallery_Chapter=".$this->curPathBack.">Back</a><b>]</b></span>");
			}
		} else {
			$this->buffer->append("<td class=gallery_td_title colspan=".$this->img_per_line."><span class=gallery_title>".$this->title."</span>");
		}
		$this->buffer->append("<br><span class=gallery_info>".$this->introTxt."</span></td></tr><tr>");
		for ($i = 0;$i<count($folders);$i++) {
			if ($i % $this->img_per_line == 0) {
				if ($i>0) {
					$this->buffer->append("</tr><tr>");
				}
			}
			$this->buffer->append("<td class=gallery_td_folder><a href=\"".$this->main_page."?CSE_Gallery_Chapter=".$this->curPath.$folders_nme[$i]."\"><img src='".$folders_tn_url[$i]."' alt=' '><br><span class=img_caption>".$folders_nme[$i]."</span></a></td>");
		}
		for ($k = 0;$k<count($images);$k++) {
			if ($i % $this->img_per_line == 0) {
				if ($i>0) {
					$this->buffer->append("</tr><tr>");
				}
			}
			$i++;
			list($width, $height, $type, $attr) = getimagesize($images[$k]);
			$this->buffer->append("<td class=gallery_td><a href=javascript:CSE_galleryPopUp(\"".$images_url[$k]."\",".$width.",".$height.")><img src='".$images_tn_url[$k]."' alt=' '><br><span class=img_caption>".$images_nme[$k]."</span></a></td>");
		}
		$this->buffer->append("</tr></table>");
		
	}
	
	function dump() {
		$this->makeGallery();
		return $this->buffer->dump();
	}

} // END CLASS CSE_gallery

class CSE_queue {

	// =====================================================
	// Priority Queue
	// =====================================================
	// This is separate as a convenience feature. 
	// You add members to the queue with a priority.
	// The queue is dumpped in increasing order of priority
	//
	// The implementation is not very effective, it is not done
	// for queues with more than some elements (not thousands).
	
	var $stack; // an associative array of elements.
	
	// creator puts the first element in.
	function CSE_queue($data, $priority) {
		$this->stack = array((" ".$priority)=>$data);
	}
	
	// query size
	function q_count() {
		return count($this->stack);
	}
	
	// add an element
	function add($data, $priority) {
		while (array_key_exists(" ".$priority, $this->stack)) {
			$priority = $priority + 0.01;
		}
		$this->stack = array_merge($this->stack,array((" ".$priority)=>$data));
	}
	
	// for convenience, return element with highest priority and delete it:
	// unused in CSE_display so untested :-D
	function pull() {
		// sort array
		ksort($this->stack, SORT_NUMERIC);
		$this->stack = array_reverse($this->stack, true);
		// take and kill last element
		$data = array_pop($this->stack);
		// return data
		return $data;
	}
	
	// dump sorted by priority with \n between records
	function dump() {
		// sort array
		ksort($this->stack, SORT_NUMERIC);
		// take content
		$dp = "";
		foreach ($this->stack as $k) {
			$dp .= $k;
		}
		return $dp;
	}	
} // END CLASS CSE_queue


class CSE_infopanel {
	// =====================================================
	// Info panels
	// =====================================================
	// This is a class that contains independant information elements,
	// that are rendered at different positions in your html page.
	// This class manages the position and the display of the panels,
	// with a title and content provided externally.
	//
	// By defult, an Infopanel will reside in a region called "LEFT". If "moveButton" is enabled,
	// It will switch in sequence between "LEFT" and "RIGHT"; can be replaced by regions defined
	// via setRegions();
	// If 2 regions exists, the button will show a right arrow when in the first region, and a left
	// arrow when in second region. If more than 2 regions exists, will always show a right arrow.
	
	
	// Button constants for bit checks on $this->hasButton: 
	// hasMinimizeButton = 0;
	// hasSmallState = 1;
	// hasPriorities = 2;
	// hasRegionSwitch = 3;
	
	// Variables
	var $region; // any region - will be outputted if the region is defined in the template.
	var $state; // mini(mized), small, large
	var $name; // no space, no special character. The name of the panel.
	var $title;
	var $content;
	var $content_small; // content dumped if state is small. Optional.
	var $region_list;
	var $reg_idx; // index of current region according to region list.
	var $priority;
	
	// preferences, all defaults are true:
	var $isInTable; // if true, insert the element in <tr><td> .... </td></tr> when dumping
					// if flase, append a <br> after the dump
	var $hasButton; // if true, has the standard button to change state and region. Otherwise not.
	var $usesCookies; // if true, does automatically uses cookies to remember its own position and state
					  // if false, IN THE FUTURE, may use a database. NOT YET IMPLEMENTED.
	var $cookiePersistence; // persistence of cookie, in seconds. Default is 1 month.
	var $cookiePath;
	var $main_page; // this is the executed page for click on a button.
	var $img_root; // location of images for buttons
	
	// creator, if cookies are enabled, cookies are "stronger" then preference input parameters.
	// to override the cookies, you have to use setInfoPanelPreferences() afterwards.
	// The only exception is the first parameter. If, in the creator, $cookies is set to false
	// then cookies will be completely ignored for this instance of the infopanel.
	// This allows to create a "fixed" instance in a given region, while letting the same
	// panel "automatized" in the standard regions.
	// Do not pass "FALSE", but "0". Cookies with a value of "false" are erased by the client! 
	function CSE_infopanel($nme, $stt = "large", $reg = "LEFT", $cookies = 1, $inTable = 1, $button = 15, $cookiePers = 2592000, $path = "/") {
		// set default region list:
		$this->region_list[] = "LEFT";
		$this->region_list[] = "RIGHT";
		$this->content = "";
		$this->name = $nme;
		$this->title = $nme; // should be changed with setTitle(), but a fitting defult...
		$this->isInTable = $inTable;
		$this->hasButton = $button;
		$this->usesCookies = $cookies;
		$this->cookiePersistence = $cookiePers;
		$this->cookiePath = $path;
		$this->img_root = "CSE_display/imazes/";
		// I assume the main page is just index.php relative to the url of the function that dumps finally the object.
		$this->main_page = "index.php"; // this should be this file;
		
		// We load the positions:
		$this->region = $reg;
		$this->state = $stt;
		$this->priority = 1;
		// We check the actual cookies
		if ($this->usesCookies) {
			// for each cookie, we read if existing, or write actual values if not existing.
			if (isset($_COOKIE["CSE_IP_".$nme."_region"])) {
				// We read the preferences
				$this->region = $_COOKIE["CSE_IP_".$nme."_region"];
			}
			// We write the actual values anyway, to prolong cookie life:
			setcookie("CSE_IP_".$nme."_region", $this->region, time()+$this->cookiePersistence, $this->cookiePath);
			
			if (isset($_COOKIE["CSE_IP_".$nme."_state"])) {
				// We read the preferences
				$this->state = $_COOKIE["CSE_IP_".$nme."_state"];
			}
			// We write the actual values anyway, to prolong cookie life:
			setcookie("CSE_IP_".$nme."_state", $this->state, time()+$this->cookiePersistence, $this->cookiePath);
			
			if (isset($_COOKIE["CSE_IP_".$nme."_priority"])) {
				// We read the preferences
				$this->priority = $_COOKIE["CSE_IP_".$nme."_priority"];
			}
			// We write the actual values anyway, to prolong cookie life:
			setcookie("CSE_IP_".$nme."_priority", $this->priority, time()+$this->cookiePersistence, $this->cookiePath);
		}
		
		// Now we need to check whether we have been "called" by one of the buttons:
		if (isset($_GET["CSE_IP_".$nme."_state"])) {
			$this->state = $_GET["CSE_IP_".$nme."_state"];
			// correct the cookie if enabled:
			if ($this->usesCookies) { setcookie("CSE_IP_".$nme."_state", $this->state, time()+$this->cookiePersistence, $this->cookiePath); }
		}
		if (isset($_GET["CSE_IP_".$nme."_region"])) {
			$this->region = $_GET["CSE_IP_".$nme."_region"];
			// correct the cookie if enabled:
			if ($this->usesCookies) { setcookie("CSE_IP_".$nme."_region", $this->region, time()+$this->cookiePersistence, $this->cookiePath); }
		}
		if (isset($_GET["CSE_IP_".$nme."_priority"])) {
			$this->priority = $_GET["CSE_IP_".$nme."_priority"];
			// correct the cookie if enabled:
			if ($this->usesCookies) { setcookie("CSE_IP_".$nme."_priority", $this->priority, time()+$this->cookiePersistence, $this->cookiePath); }
		}
		
		// find region index:
		$this->reg_idx = array_search($this->region, $this->region_list);
	}
	
	// check whether a bit is set in the field. Return 0 (not set) or 1 (set)
	function CSE_isSet($bit, $field) {
		$mask = pow(2,$bit);
		if (($mask&$field)==$mask) {
			return 1;
		} else {
			return 0;
		}
	}
	
	function setMainPage($url) {
		$this->main_page = $url;
	}
	
	function setImageRoot($root) {
		$this->img_root = $root;
	}
	
	// May be useful...
	function getPriority() {
		return $this->priority;
	}
	
	function setPriority($prio, $writeCookie = true) {
		$this->priority = $prio;
		if (($writeCookie) & ($this->usesCookies)) {
			setcookie("CSE_IP_".$this->name."_priority", $this->priority, time()+$this->cookiePersistence, $this->cookiePath);
		}
	}
	
	// this functions forces a given position and region, without overwriting the cookies:
	function positionInfoPanel($stt = "large", $reg = "LEFT") {
		$this->region = $reg;
		$this->state = $stt;
		// find region index:
		$this->reg_idx = array_search($this->region, $this->region_list);
	}
	
	// this functions forces a given position and region, if no cookie exists - in essence, it is a default behavior.
	function defaultpositionInfoPanel($stt = "mini", $reg = "LEFT") {
		if (!isset($_GET["CSE_IP_".$this->name."_state"]) and !isset($_COOKIE["CSE_IP_".$this->name."_state"])) {
			$this->state = $stt;
		}
		if (!isset($_GET["CSE_IP_".$this->name."_region"]) and !isset($_COOKIE["CSE_IP_".$this->name."_region"])) {
			$this->region = $reg;
			// find region index:
			$this->reg_idx = array_search($this->region, $this->region_list);
		}
	}
	
	// overrides cookie info WITHOUT overwriting them.
	function setInfoPanelPreferences($cookies = 1, $inTable = 0, $button = 15, $cookiePers = 2592000) {
		$this->isInTable = $inTable;
		$this->hasButton = $button;
		$this->usesCookies = $cookies;
		$this->cookiePersistence = $cookiePers;
	}
	
	// overrides default regions "LEFT" and "RIGHT".
	function setRegions($regions) {
		$this->region_list = $regions;
		// find region index:
		$this->reg_idx = array_search($this->region, $this->region_list);
	}
	
	// Set display title
	function setTitle($TextTitle) {
		$this->title = $TextTitle;
	}
	
	function appendContent($data) {
		$this->content .= $data;
	}
	
	function prependContent($data) {
		$this->content = $data . $this->content;
	}
	
	function appendContentSmall($data) {
		$this->content_small .= $data;
	}
	
	function prependContentSmall($data) {
		$this->content_small = $data . $this->content;
	}
	
	function getRegion() {
		return $this->region;
	}
	
	// dump function
	function dump() {
		$buffer = "<div><table class='infobox_".$this->state."'>\n";
		if ($this->isInTable) { $buffer = "<tr><td>".$buffer; }
		
		// OK, we are in the div. Let's go for the "title" bar:
		if (!$this->hasButton) {
			$buffer .= "<tr><td class='infobox_title' colspan=2>".$this->title."</td></tr>";
		} else {
			$buffer .= "<tr><td class='infobox_title'>".$this->title."</td>";
			$buffer .= "<td class=icon_region>";
			// Put the state buttons, remembering:
			// Button constants for bit checks on $this->hasButton: 
			// hasMinimizeButton = 0;
			// hasSmallState = 1;
			// hasPriorities = 2;
			// hasRegionSwitch = 3;
			if ($this->state == "mini") {
				$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=large'><img class=icon src='".$this->img_root."open.gif' title='Enlarge' alt='Enlarge'></a>";
				if ($this->CSE_isSet(1, $this->hasButton)) { $buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=small'><img class=icon src='".$this->img_root."arrow3_s.gif' title='Small' alt='Small'></a>"; }
			} elseif ($this->state == "small") {
				$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=large'><img class=icon src='".$this->img_root."arrow3_s.gif' title='Enlarge' alt='Enlarge'></a>";
				if ($this->CSE_isSet(0, $this->hasButton)) { $buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=mini'><img class=icon src='".$this->img_root."arrow3_n.gif' title='Minimize' alt='Minimize'></a>"; }
			} elseif ($this->state == "large") {		
				if ($this->CSE_isSet(1, $this->hasButton)) { $buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=small'><img class=icon src='".$this->img_root."arrow3_n.gif' title='Small' alt='Small'></a>"; }
				if ($this->CSE_isSet(0, $this->hasButton)) { $buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_state=mini'><img class=icon src='".$this->img_root."close.gif' title='Minimize' alt='Minimize'></a>"; }
			} // no else. If unknown, no button.
			// Put the priority button
			if ($this->CSE_isSet(2, $this->hasButton)) { 
				$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_priority=".($this->priority-1.1)."'><img class=icon src='".$this->img_root."arrow2_n.gif' title='Higher' alt='Higher'></a>";
				$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_priority=".($this->priority+1.1)."'><img class=icon src='".$this->img_root."arrow2_s.gif' title='Lower' alt='Lower'></a>";
			}
			// Put the region button
			if ($this->CSE_isSet(3, $this->hasButton)) {
				if (count($this->region_list)==2) {
					if ($this->reg_idx == 0) {
						$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_region=".$this->region_list[1]."'><img class=icon src='".$this->img_root."arrow1_e.gif' title='Move' alt='Move'></a>";
					} else {
						$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_region=".$this->region_list[0]."'><img class=icon src='".$this->img_root."arrow1_w.gif' title='Move' alt='Move'></a>";
					}
				} else {
					if (($this->reg_idx+1)<count($this->region_list)) {
						$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_region=".$this->region_list[$this->reg_idx+1]."'><img class=icon src='".$this->img_root."arrow1_e.gif' title='Move' alt='Move'></a>";
					} else {
						$buffer .= "<a href='".$this->main_page."?CSE_IP_".$this->name."_region=".$this->region_list[0]."'><img class=icon src='".$this->img_root."arrow1_e.gif' title='Move' alt='Move'></a>";
					}
				}
			}
			// close button region:
			$buffer .= "</td></tr>";
		}
		// Now we are in the body part...
		if ($this->state != "mini") {
			$buffer .= "<tr><td colspan=2>";
			if ($this->state == "large") {
				// just dump
				$buffer .= $this->content;
			} else {
				// If a small content is provided, dump:
				if ($this->content_small != "") {
					$buffer .= $this->content_small;
				} else {
					// we could just dump with smaller font. But let's try a layout in 2 columns:
					
					
					// ... Not implemented yet...
					
					// just dump
					$buffer .= $this->content;
				}
			}	
		
			$buffer .= "</td></tr></table></div>\n";
		} else {
			// we don't need no body...
			$buffer .= "</table></div>\n";
		}
		if ($this->isInTable) { $buffer .= "</td></tr>\n"; } else { $buffer .= "<br>\n"; }
		
		return $buffer;
	}
	
	

} // END CLASS CSE_infopanel


class CSE_display {

	// =====================================================
	// Variables
	// =====================================================

	var $js_functions; // this associative array will hold a collection of js functions, put in *{JS}* region (in <head>)
	var $onLoad; // this will register js functions that must be called onLoad, put in *{ONLOAD}*
	var $title; // this will be put in the *{TITLE}* region and in the page title
	var $js_files; // allow to include external javascripts. Printed out also in *{JS}* region.
	var $body; // this will be put in the *{BODY}* region; this is a CSE_buffer, so it has many layers if required.
	var $body_title; // Top of body, in *{BODY_TITLE}*
	var $body_foot; // bottom of body, in *{BODY_FOOT}*
	var $footer; // this will be put in the *{FOOTER}* region, if any...
	var $copyright; // this will be put in the *{COPY}* region
	var $regions; // this is an associative array of priority lists. Will be put in the *{ __ }* region corresponding to the key.
	var $errors; // contains error messages that should not come. Printed on the top, without style! -- this is not the location for
				 // "normal" error messages, that should be printed out within the *{BODY}*.
	public $infopanels; // an array of CSE_infopanel objects; they will dump themselves in the correct region AFTER the rest of the content
			   		    // of the region.
	
	var $bodies; // maximum number of layers in body.
	
	var $blogs; // contain independant article and manage the layout
	var $gallery; // contain thumbnail gallery
	var $more_txt; // text to display for link to main article ("read more")
	var $writtenby; // text for "written by"
	
	var $template; // address of template file
	
	var $css; // address of CSS file
	
	var $path_to_CSE_Display; // path to this file. Only required for ajax functionality
	
	// =====================================================
	// CREATOR
	// =====================================================
	
	function CSE_display($n_css = "CSE_display/CSE_CSS.css",$n_template = "CSE_display/CSE_template.txt") {
		// called on "new", constructor.
		
		// use specified or default template and CSS:
		$this->template = $n_template;
		$this->css = $n_css;
		
		// write default copyright, hoping that people will forget to change it :-D
		$this->copyright = "Made with CSE_display by <a href=www.cavin.name>CSE</a>!";
		
		// create the buffer for the body:
		$this->body = new CSE_buffer;
		$this->bodies = 1000;
		
		$this->errors = "";
		
		$this->more_txt = "Read more...";
		$this->writtenby = "by";
		
	}

	function getCSEDisplayInfo() {
		return "<a href=\"http://www.cavin.name/softs/\">CSE_Display</a>, last modified: ".date("d-m-Y",getlastmod());
	}
	
	
	// =====================================================
	// Layout and style
	// =====================================================
	
	// Change the template
	function setTemplate($newTemplate) {
		$this->template = $newTemplate;
	}
	
	// Change the CSS
	function setCSS($newCSS) {
		$this->css = $newCSS;
	}
	
	// =====================================================
	// Javascript
	// =====================================================
	//
	// The first parameter is the name of the function.
	// It should include the parenthesis and parameters.
	// The second parameter is the body. It should not include
	// the curly brackets around the function itself, in order to be
	// able to prepend and append more JS commands!
	//
	// Example:
	//
	// registerJSfunction("checkChecks(mx_checked)","cur_checked = 0;\nif (cur_checked >= mx_checked) {\n myform.F_2.disabled = false;\n}");
	//
	// Will print out when the page is generated (if it was the only function registered) - see that
	// the main function curly brackets have been added:
	//
	// <script type='text/javascript'>
	// <!--
	//
	// function checkChecks(mx_checked) {
	// cur_checked = 0;
	// if (cur_checked >= mx_checked) {
	//  myform.F_2.disabled = false;
	// }
	// }
	//
	// //-->
	// </script>\n
	
	// add an external javascript file
	function addExternalJS($f_name) {
		$this->js_files[] = "<script type='text/javascript' src='".$f_name."'></script>\n";
	}
	
	// force a javascript command to be outputted
	function addJScommand($command) {
		$this->js_files[] = "<script type='text/javascript'>".$command."</script>\n";
	}
	
	// Create new JS function
	function registerJSfunction($f_name,$f_txt) {
		$this->js_functions[$f_name]=$f_txt;
	}
	
	// Append code to a given function
	function appendJSfunction($f_name,$t_txt) {
		$this->js_functions[$f_name].=$f_txt;
	}
	
	// Prepend code to a given function (useful e.g. for variable declaration)
	function prependJSfunction($f_name,$t_txt) {
		$this->js_functions[$f_name]=$f_txt.$this->js_functions[$f_name];
	}
	
	// Remove a given function
	function killJSfunction($f_name) {
		unset($this->js_functions[$f_name]);
	}
	
	// Dumps the js and return it as a string.
	function dumpJS() {
		if (count($this->js_files)>0) {
			$dump = implode("\n",$this->js_files);
		} else {
			$dump = "";
		}
		if (count($this->js_functions)>0) {
			$dump .= "\n<script type='text/javascript'>\n<!--\n\n";
 			foreach ($this->js_functions as $ttle => $ttxt) {
				$dump .= "function ".$ttle." {\n".$ttxt."\n}\n\n";
  			}
			$dump .= "//-->\n</script>\n";
 		}
		return $dump;
	}
	
	// Register a function as being executed on load of the page.
	// f_name is the exact call to the function, e.g. "checkChecks(2);"
	// Don't forget the semicolon after each command!
	// TAKE CARE: String parameters MUST be passed between double quotes ". No single quotes!
	// e.g. registerJSonLoad("myCoolName(\"Abdul\",\"Ar\",\"Hamlid\");");
	function registerJSonLoad($f_name) {
		$this->onLoad .= $f_name;
	}
	
	// Same, but added function will be executed BEFORE the other functions already
	// registered.
	function registerJSonLoadBefore($f_name) {
		$this->onLoad = $fname . $this->onLoad;
	}
	
	// You should never need to call this. It dumps the onLoad and return it as a string.
	function dumpOnLoad() {
		return "onLoad='".$this->onLoad."'";
	}

	// =====================================================
	// AJAX Support
	// =====================================================
	//
	// This enables ajax support in the generated web page.
	// AJAX require both client and server side scripts, so you
	// will have to provide both. On client side, some functions
	// here will help you. On server side, you must write the
	// necessary scripts. Some examples are provided.
	
	function activateAJAX($path_to_CSE_Display = "CSE_display/", $path_to_handler = "") {
		if ($path_to_handler == "") {
			$this->addJScommand("var CSE_ajax_script = \"".$path_to_CSE_Display."CSE_ajax.php\";");
		} else {
			$this->addJScommand("var CSE_ajax_script = \"".$path_to_handler."CSE_ajax.php\";");			
		}
		$this->path_to_CSE_Display = $path_to_CSE_Display;
		$this->addExternalJS($path_to_CSE_Display."CSE_ajax.js");
	}
	
	// file is the .php file server side that will answer to the request
	//		if empty, will try to call the function within CSE_ajax.php, useful for some default functions.
	// command is the name of the function within this file
	// parameter is the parameters to send to the function in form of 
	//			id1<:>property1<|>id1<:>property2<|>id2<:>property3
	//		you can directly specify parameters (as opposed as dynamically assigned properties
	//			of HTML elements) using an empty id, e.g. id1<:>property1<|><:>param1<|><:>param2<|>id2<:>property2
	//		if several parameters are specified with the pipe |, then the server-side script
	//			wil recieve them in an array.
	// target is the optional id:property where returned information should be outputted.
	//
	// it returns a javascript that must be added in the event desired,
	// onClick for buttons, onKeyUp for auto-suggest in texts, etc...
	//
	// A typical call will be (¿ is actually ?) for an auto-suggest:
	//		<textarea id=text1 onChange='<?php ajaxIt("myfile.php", "myfunction", "text1:value", "text2:value"); ¿>'></textarea><br>
	//		<textarea id text2></textarea>
	//
	//	 myfunction will recieve as parameter the content of text1, and the fact that the response should go in text2:value.
	//	 it is its responsability to actually write something in text2.value using some helper functions...
	//
	// Another typical call will be (¿ is actually ?) for a button that change a text field:
	//		<textarea id=text1></textarea><br>
	//		<input type=button onclick='<?php ajaxIt("myfile.php", "myfunction", "text1:value", "text1:value"); ¿>'>
	function ajaxIt($file, $command, $parameter, $target, $debug=0) {
		if ($file=="") { $file = "CSE_ajax.php"; }
		$str = "CSE_ajax.run('".$file."', '".$command."', '";
		// now we may need several parameters:
		$temp_str = explode("<|>", $parameter);
		for ($i=0;$i<count($temp_str);$i++) {
			if ($i>0) { $str.="<|>"; }
			$couple = explode("<:>", $temp_str[$i]);
			if ($couple[0]=="") {
				// direct parameter
				$str.=$couple[1];
			} else {
				// fetch a property
				$str.="'+CSE_ajax.getProperty('".$couple[0]."', '".$couple[1]."')+'";
			}
		}
		$str.="', '".$target."',".$debug.")";
		return $str;
	}

	// =====================================================
	// Main Elements
	// =====================================================
	
	// Set Title (replaces)
	function setTitle($txt) {
		$this->title = $txt;
	}
	
	// Set footer (replaces)
	function setFooter($txt) {
		$this->footer = $txt;
	}
	
	// Append to footer
	function appendFooter($txt) {
		$this->footer .= $txt;
	}
	
	// Prepend to footer
	function prependFooter($txt) {
		$this->footer = $txt.$this->footer;
	}
	
	// Set copyright string
	function setCopyright($txt) {
		$this->copyright = $txt;
	}
	
	// If an error message must be outputted on the top of the window, OUTSIDE the normal layout.
	function addError($txt) {
		$this->errors .= $txt."<br>\n";
	}
	

	// =====================================================
	// Main Body
	// =====================================================
	//
	// It is a buffer (see above). Can be used as default layer
	// setBody("txt"), appendBody("txt")...
	// or with layers:
	// setBody("txt",ID), appendBody("txt",ID)...
	// all layers will be dumped in the body space in the order of ID.
	
	// But first body title and footer	
	function setBodyTitle($txt) {
		$this->body_title = $txt;
	}
	
	function setBodyFooter($txt) {
		$this->body_foot = $txt;
	}
	
	
	// Set Body (replaces)
	function setBody($txt, $id=0) {
		if ($id == 0) {
			$this->body->set($txt);
		} else {
			if ($this->body->exist($id)) {
				$this->body->set($txt, $id);
			} else {
				// make new buffer elements until it exists
				// this is a "feature". I can write directly to
				// body(3) and trust that (default) 0, 1, 2 will be
				// outputted before.
				// as a safety net, we limit ourself to $bodies positions
				if (!is_numeric($id)) { die("Body Layers need an integer reference."); }
				$id = intval($id);
				if ($id >= $this->bodies) { die("Not so many body layers authorized (you required ".$id." layers)."); }
				// now we go
				$new_id = 0;
				while ($new_id < $id) {
					$new_id = $this->body->newBuffer();
					//echo $id."&lt;".$new_id;
				}
				// and we put the string
				$this->body->set($txt, $id);
			}
		}
	}
	
	// Append Body
	function appendBody($txt, $id=0) {
		if ($id == 0) {
			$this->body->append($txt);
		} else {
			if ($this->body->exist($id)) {
				$this->body->append($txt, $id);
			} else {
				// in line with PHP philosophy (never crash), we don't fail, but create...
				$this->setBody($txt, $id);
			}
		}
	}
	
	// Prepend Body
	function prependBody($txt, $id=0) {
		if ($id == 0) {
			$this->body->prepend($txt);
		} else {
			if ($this->body->exist($id)) {
				$this->body->prepend($txt, $id);
			} else {
				// in line with PHP philosophy (never crash), we don't fail, but create...
				setBody($txt, $id);
			}
		}
	}
	
	// Switch order for output. This one takes 2 ID and switch their content, effectively
	// switching the order they will be dumpped:
	function swapBodyLayers($id1, $id2) {
		$temp = $this->body->dump($id1);
		$this->body->set($this->body->dump($id2), $id1);
		$this->body->set($temp, $id2);
	}

	// This function takes the content of two layers and append the second one to the first one
	// the second one is emptied.
	function appendBodyLayers($id1, $id2) {
		$this->body->append($this->body->dump($id2), $id1);
		$this->body->set("", $id2);
	}
	
	// dump body, optionally a single layer
	function dumpBody($id=-1) {
		$dump = "";
		if ($id == -1) {
			$id = 0;
			while ($this->body->exist($id)) {
				$dump .= "\n".$this->body->dump($id);
				$id++;
			}
		} else {
			if ($this->body->exist($id)) {
				$dump = $this->body->dump($id);
			} else {
				$dump = "";
			}
		}
		return $dump;
	}
	
	// =====================================================
	// Blogs
	// =====================================================
	//
	// Alternatively to a standard body, you can insert several independant "articles" and let
	// CSE_Display organize them for you. The content of Blogs (whch is a priority list) will be
	// dumped AFTER any content put in the body with the default setBody(), appendBody() or other
	// functions, but before numbered body elements.
	// Similarly, you can insert a picture gallery.
	
	function registerBlogs($title, $data, $priority=0, $date = "", $link = "", $author = "") {
		// create record
		if ($link != "") {
			$txt = "<span class=blog_title><a href='".$link."'>".$title."</a></span>";
		} else {
			$txt = "<span class=blog_title_stat>".$title."</span>";
		}
		$txt .= "<span class=blog_head>";
		if ($date != "") { $txt .= "(".$date.")"; }
		if ($author != "") { $txt .= " ".$this->writtenby." <span class=blog_author>".$author."</span>"; }
		$txt .= "</span><br><div class=blog_body>".$data."</div>";
		if (count($this->blogs)>0) {
			// already defined
			$this->blogs->add($txt,$priority);
		} else {
			$this->blogs = new CSE_queue($txt, $priority);
		}
		
	}
	
	// Language functions
	function setBlogReadMore($txt) {
		$this->more_txt = $txt;
	}
	
	// Language functions
	function setBlogBy($txt) {
		$this->writtenby = $txt;
	}
	
	function dumpBlogs() {
		// escape if unused
		if (!isset($this->blogs)) { return ""; }
		if ($this->blogs->q_count()<1) { return ""; }
		
		$str = "<table width='100%'>";
		// if only 2, keep stacking
		if ($this->blogs->q_count()<3) {
			$str .= "<tr><td colspan=2 class=blog_col>";
			$str .= $this->blogs->pull();
			$str .= "</td></tr>\n";
			if ($this->blogs->q_count()>1) {
				$str .= "<tr><td colspan=2 class=blog_col>";
				$str .= $this->blogs->pull();
				$str .= "</td></tr>\n";
			}
		}
		
		// if odd, first gets 2 columns...
		if ($this->blogs->q_count()%2==1) {
			$str .= "<tr><td colspan=2 class=blog_col>";
			$str .= $this->blogs->pull();
			$str .= "</td></tr>\n";
		}
		
		
		// now 2 by 2
		while ($this->blogs->q_count()>1) {
			if ($this->blogs->q_count()>3) {
				$comp = "";
			} else {
				$comp = "_bot";
			}
			$str .= "<tr><td width='50%' class=blog_col_left".$comp.">";
			$str .= $this->blogs->pull();
			$str .= "</td>\n";
			$str .= "<td width='50%' class=blog_col".$comp.">";
			$str .= $this->blogs->pull();
			$str .= "</td></tr>\n";
		}
		$str .= "</table>";
		return $str;
	}
	
	
	// =====================================================
	// Gallery
	// =====================================================
	//
	// Alternatively to a standard body, you can insert a picture gallery.
	// The gallery system supports subfolders automatically, using
	// GET forms.
	
	function registerGallery($dir, $title="") {
		if (!isset($this->gallery)) { 
			$this->gallery = new CSE_gallery($dir, $title);
			$this->registerJSfunction("CSE_galleryPopUp(URL,x,y)","day = new Date();\nid = day.getTime();\nleft=((screen.width-x)/2);\ntop=((screen.height-y)/2);\neval(\"page\" + id + \" = window.open(URL, '\" + id + \"', 'toolbar=0,scrollbars=0,location=0,statusbar=0,menubar=0,resizable=0,width=\"+x+\",height=\"+y+\",left = \"+left+\",top = \"+top+\"');\");");
		} else {
			// only 1 gallery supported per page. Do nothing.
		}
	}
	
	function dumpGallery() {
		// escape if unused
		if (!isset($this->gallery)) { return ""; }
		return $this->gallery->dump();
	}


	// =====================================================
	// Regions
	// =====================================================
	//
	// This is flexibility puuur. You can define custom locations. Like your left_column or
	// anything else. You dump text in it. In the template, when a marker with the same name
	// is found, the text is dumpped. So you can have infinite regions!
	// In your template, you define a region with the *{ and }* markers: *{MY_REGION}*. When the
	// marker is found, the region is dumped.
	//
	// There is more. Each region is a priority queue, so you can append in the region different
	// things that will be outputted in the order of their priority.
	//
	// SOON TO COME:
	// By the way -- you can define a region in your body, if you want to. Then you get a priority
	// queue feature in your body. How? define a new region with a label (i.e. "PRIO_ROWS").
	// put in your region things as they come:
	// $mydisplay->newRegion("PRIO_ROWS", "<td>23</td>", 23);
	// $mydisplay->addRegion("PRIO_ROWS", "<td>13</td>", 13);
	// $mydisplay->addRegion("PRIO_ROWS", "<td>31</td>", 31);
	//
	// Now in your body, you append the marker for a region:
	// $mydisplay->appendBody("<table><tr>*{PRIO_ROWS}*</tr></table>");
	//
	// The result? A sorted table:
	//	13	23	31
	//
	// This works ONLY in body. All other texts are not parsed before output. The body is parsed just 
	// before output. You can write the marker, then keep filling the region until you dump the
	// whole HTML.
	
	
	// register a new region
	function newRegion($label, $content="", $priority=0) {
		$this->regions[$label] = new CSE_queue($content, $priority);
	}
	
	// append in a region, create if necessary
	function appendRegion($label, $content, $priority=0) {
		if (!$this->regionExists($label)) {
			$this->newRegion($label, $content, $priority);
		} else {
			$this->regions[$label]->add($content, $priority);
		}
	}
	
	// dump region, you should never need this
	function dumpRegion($label) {
		return $this->regions[$label]->dump();
	}
	
	// does region exist?
	function regionExists($label) {
		if (isset($this->regions)) {
			if (array_key_exists($label,$this->regions)) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	
	// =====================================================
	// Info Panels
	// =====================================================
	// You can register independant snipets of code, that manage themselves their display preferences.
	// On the very basic, you can use them to display themselfes in pre-defined regions *{LEFT}* and
	// *{RIGHT}*. Advanced features allow to define your own regions (arbitrary number)...
	// By default, the panels manage their own cookies. If the user has no cookies, panels will simply
	// always be dumped in their default region.
	// A region does not need to be registered for a panel to be displayed there...
	// If a region is registered and has a content, this content is displayed before any panel.
	
	// add a panel, and return its ID for future reference.
	function addPanel($nme, $stt = "large", $reg = "LEFT", $cookies = 1, $inTable = 1, $button = 15, $cookiePers = 2592000, $path = "/") {
		// Button constants (sum of 2^cte) 
		// hasMinimizeButton = 0;
		// hasSmallState = 1;
		// hasPriorities = 2;
		// hasRegionSwitch = 3;
		$this->infopanels[] = new CSE_infopanel($nme, $stt, $reg, $cookies, $inTable, $button, $cookiePers, $path);
		return (count($this->infopanels)-1);
	}
	
	// These serves as transmission for functions of the class CSE_infopanel supported via the CSE_display.
	// all can be called as myDisplay->infopanels[ID]->class_functions(), because infopanels is public.
	function IP_setTitle($id, $title) {
		$this->infopanels[$id]->setTitle($title);
	}
	function IP_setPreferences($id, $cookies = 1, $inTable = 0, $button = 15, $cookiePers = 2592000) {
		// Button constants (sum of 2^cte) 
		// hasMinimizeButton = 0;
		// hasSmallState = 1;
		// hasPriorities = 2;
		// hasRegionSwitch = 3;
		$this->infopanels[$id]->setInfoPanelPreferences($cookies, $inTable, $button, $cookiePers);
	}
	function IP_appendBody($id, $data) {
		$this->infopanels[$id]->appendContent($data);
	}
	function IP_prependBody($id, $data) {
		$this->infopanels[$id]->prependContent($data);
	}
	function IP_appendBodySmall($id, $data) {
		$this->infopanels[$id]->appendContentSmall($data);
	}
	function IP_prependBodySmall($id, $data) {
		$this->infopanels[$id]->prependContentSmall($data);
	}
	
	// These functions will be applied on all info panels already defined. You should call them AFTER having defined
	// ALL your panels.
	function IP_setRegions($regions) {
		foreach ($this->infopanels as $panel) {
			$panel->setRegions($regions);
		}
	}
	function IP_setMainPage($url) {
		foreach ($this->infopanels as $panel) {
			$panel->setMainPage($url);
		}
	}
	function IP_setImageRoot($root) {
		foreach ($this->infopanels as $panel) {
			$panel->setImageRoot($root);
		}
	}
	function IP_setPrefAll($button = 15, $inTable = 0, $cookies = 1, $cookiePers = 2592000) {
		foreach ($this->infopanels as $panel) {
			$panel->setInfoPanelPreferences($cookies, $inTable, $button, $cookiePers);
		}
	}
	
	// sorts the info panels with priority.
	// Rewrite panels with monotoneous integer priorities...
	function organizeInfoPanels() {
	
		$i=0;
		if (count($this->infopanels)==0) {
			return;
		}
		foreach ($this->infopanels as $panel) {
			$prio[$i] = $panel->getPriority();
			$i++;
		}
		$ok=asort($prio);
		$i = 0;
		foreach ($prio as $panelID => $priority) {
			if ($this->infopanels[$panelID]->getPriority() != $i) {
				$this->infopanels[$panelID]->setPriority($i);
			}
			$i++;
			// priorities are given.
			// Now we want a display in this order, so we rearrange the array:
			$temp[] = $this->infopanels[$panelID];
		}
		
		// and we transfer:
		$this->infopanels = $temp;
		
	
		//print_r($this->infopanels);
	}

	// =====================================================
	// The output function
	// =====================================================
	// When happy with your content... dump it!
	// This functions generates a complete HTML file, provided
	// the template is complete. Therefore you should theoretically not output
	// anything outside this function, so no echo in your code.
	// Note: you can use addError if you want to output an error message.
	
	// This dumps to a string. You see why? I leave you liberty to combine several
	// thus generated HTML snippets. Of course, only one need a template with the header
	// only one can have onLoad, but you can use the same object for e.g. preparing
	// complex <div>s, with a simple template like for instance a 2 colums:
	// $myDivDisplay->dump2str("<div><table><tr><td width=50%>*{LEFT}*</td><td>*{BODY}*</td></tr></table></div>");
	// $myDisplay->appendBody($myDivDisplay);
	// this uses 2 objects, $myDivDisplay and $myDisplay, obviously.
	function dump2str($simpleTemplate="") {
		// before everything, we sort out all the CSE_infopanels we might have, regardless of region:
		$this->organizeInfoPanels();
		
		// and now go, go, go!
		if ($simpleTemplate=="") {
			// read the template:
			$myHTML = file_get_contents($this->template);
		} else {
			// use the template given.
			$myHTML = $simpleTemplate;
		}
		
		// prepend potential errors:
		if ($this->errors != "") {
			$myHTML = $this->errors."\n<br>\n".$myHTML;
		}
		
		// Now we parse the string for our marker: *{  }*
		$offset = 0;
		while (preg_match("/\*\{([^}]*)\}\*/", $myHTML, $matches, PREG_OFFSET_CAPTURE, $offset)) { // change to WHILE
			// in principle, we have in $matches[1][0] the region and in $matches[1][1] the offset of this find.
			switch ($matches[1][0]) {
				case "TITLE":
					$myHTML = str_replace("*{TITLE}*",$this->title,$myHTML);
					break;
				case "FOOTER":
					$myHTML = str_replace("*{FOOTER}*",$this->footer,$myHTML);
					break;
				case "CSS":
					$myHTML = str_replace("*{CSS}*",$this->css,$myHTML);
					break;
				case "JS":
					$myHTML = str_replace("*{JS}*",$this->dumpJS(),$myHTML);
					break;
				case "ONLOAD":
					$myHTML = str_replace("*{ONLOAD}*",$this->dumpOnLoad(),$myHTML);
					break;
				case "COPYRIGHT":
					$myHTML = str_replace("*{COPYRIGHT}*",$this->copyright,$myHTML);
					break;
				case "BODY_TITLE":
					$myHTML = str_replace("*{BODY_TITLE}*",$this->body_title,$myHTML);
					break;
				case "BODY_FOOT":
					$myHTML = str_replace("*{BODY_FOOT}*",$this->body_foot,$myHTML);
					break;
				case "BODY":
					// We promised to parse body for queues! Still missing!
					$buffer = $this->dumpBody(0);
					$this->setBody("",0);
					// well I changed my mind. Use "blogs" instead :-D
					$buffer .= $this->dumpBlogs();
					// why not put a nice gallery:
					$buffer .= $this->dumpGallery();
					// and dump the rest of body:
					$buffer .= $this->dumpBody();
					$myHTML = str_replace("*{BODY}*",$buffer,$myHTML);
					break;
				default:
					// This is any custom region
					// We must check if the region has been defined
					$buffer = "";
					if (count($this->regions)>0 && array_key_exists($matches[1][0],$this->regions)) {
						$buffer = $this->dumpRegion($matches[1][0]);
					}
					// we prepare the dump of any CSE_infopanel that want to be in this region:
					if (count($this->infopanels)>0) {
						foreach ($this->infopanels as $panel) {
							//echo $panel->getRegion()." : ".$matches[1][0]." - ".($panel->getRegion() == $matches[1][0]);
							if ($panel->getRegion() == $matches[1][0]) {
								//echo $panel->dump();
								$buffer .= $panel->dump();
							}
						}
					}
					// and we replace in template
					$myHTML = str_replace("*{".$matches[1][0]."}*",$buffer,$myHTML);
			}
		}
		
		
		
		return $myHTML;
	}
	
	// This dumps to the screen
	function dump() {
		echo $this->dump2str();
	}


} // END CLASS CSE_display